/** @file
 * @brief PadicoTM tasks, tasklets, and event channels
 * @note shouldn't be used directly: #include <Padico/PadicoTM.h> instead.
 * @ingroup TaskManager
 */

#ifndef TM_TASKS_H
#define TM_TASKS_H

/** @defgroup PadicoTasks API: PadicoTM tasks, tasklets, and event channels
 * @ingroup TaskManager
 */

/** @addtogroup PadicoTasks
 * @{
 */


/* ******************************************************* */
/* *** padico_tasklet */

/** An abstract tasklet descriptor
 * @note struct padico_tasklet_s is private (implementation-dependant) 
 * and needn't be exposed in the API 
 */
typedef struct padico_tasklet_s*padico_tasklet_t;

typedef void (*padico_event_listener_t)(void*event);

/** Create a new tasklet */
padico_tasklet_t padico_tasklet_new(const char*name);
/** Stop and destroy a tasklet */
void padico_tasklet_destroy(padico_tasklet_t t);
/** Add an event listener to the tasklet */
void padico_tasklet_subscribe(padico_tasklet_t t, padico_event_listener_t f);
/** Remove an event listener from a tasklet */
void padico_tasklet_unsubscribe(padico_tasklet_t t, padico_event_listener_t f);
/** schedule a new task item to the tasklet */
void padico_tasklet_schedule(padico_tasklet_t t, void*item);
void padico_tasklet_acquire(padico_tasklet_t t);
void padico_tasklet_release(padico_tasklet_t t);
void padico_tasklet_flush(padico_tasklet_t t);


/** @} */

#endif /* TM_TASKS_H */

