/** @file
 * helper to bootstrap PadicoTM as a library
 */

#ifndef PADICO_BOOTLIB_H
#define PADICO_BOOTLIB_H

#include <Padico/Puk.h>
#include <Padico/PadicoTM.h>
#include <stdlib.h>

/** initialize PadicoTM as a library */
static inline puk_mod_t padico_lib_init(void)
{
  puk_mod_t boot_mod = NULL;
  if(puk_opt_parse_bool(getenv("PADICO_PRELOAD")))
    {
      /* PadicoTM was preloaded by launcher */
      puk_mod_t tm_mod = puk_mod_getbyname("PadicoTM");
      assert(tm_mod != NULL);
      puk_mod_lock_use(tm_mod); /* lock PadicoTM module to prevent auto-unload */
    }
  else
    {
      /* manually load PadicoTM */
      padico_puk_add_path(PADICOTM_ROOT);
      padico_rc_t rc = padico_puk_mod_resolve(&boot_mod, "PadicoBootLib");
      if((boot_mod == NULL) || padico_rc_iserror(rc))
	{
	  padico_fatal("PadicoBootLib: cannot resolve module PadicoBootLib.\n");
	}
      rc = padico_puk_mod_load(boot_mod);
      puk_mod_lock_use(boot_mod); /* lock PadicoBootLib module to prevent auto-unload */
      if(padico_rc_iserror(rc))
	{
	  padico_fatal("PadicoBootLib: cannot load module PadicoBootLib.\n");
	}
      puk_mod_t tm_mod = NULL;
      padico_rc_delete(rc);
      rc = padico_puk_mod_resolve(&tm_mod, "PadicoTM");
      if(!padico_rc_iserror(rc))
	{
	  puk_mod_add_dep(tm_mod, boot_mod);
	}
      padico_rc_delete(rc);
    }
  return boot_mod;
}

static inline void padico_lib_finalize(puk_mod_t boot_mod)
{
  puk_mod_unlock_use(boot_mod); /* release lock, allow unload */
  /*
    padico_rc_t rc = padico_puk_mod_unload(boot_mod);
    if(padico_rc_iserror(rc))
    {
    padico_warning("PadicoBootLib: error while unloading.\n");
    }
    padico_rc_delete(rc);
  */
}

#endif /* PADICO_BOOTLIB_H */
