/** @file Launcher stub for 'LegacyBinary' service.
 */

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>

#include <Padico/Puk.h>
#include <Padico/PadicoTM.h>


static void padico_legacy_init(void) __attribute__((constructor));
static void padico_legacy_finalize(void) __attribute__((destructor));


/* ********************************************************* */

static void padico_legacy_init(void)
{
  if(puk_opt_parse_bool(getenv("PADICO_VERBOSE")))
    {
      const char*legacy_binary = getenv("PADICO_LEGACY");
      fprintf(stderr, "LegacyBinary: starting preload for %s.\n", legacy_binary);
    }
  if(puk_opt_parse_bool(getenv("PADICO_VALGRIND")))
    {
      fprintf(stderr, "# PadicoTM: valgrind detected. No preinit done.\n");
      unsetenv("PADICO_VALGRIND");
#ifdef PUK_VG_NOOP
      fprintf(stderr, "# PadicoTM: WARNING- running on valgrind *without valgrind integration*.\n");
      sleep(3);
#endif
      return;
    }
  puk_abi_disable_preload();

  padico_puk_init(0, NULL);
  padico_tm_init();
  
  padico_tm_ready_wait();
}

static void padico_legacy_finalize(void)
{
  padico_tm_finalize();

  /*  padico_puk_shutdown(); */
  
  if(puk_opt_parse_bool(getenv("PADICO_VERBOSE")))
    {
      fprintf(stderr, "LegacyBinary: exit.\n");
    }
}

