/** @file psm-discover.c
 *  @brief Intel PSM detection
 */

#include <Padico/Puk.h>
#include <Padico/Module.h>
#include <Padico/Topology.h>
#include <psm.h>

PADICO_MODULE_DECLARE(PSM_discover);
PADICO_MODULE_INIT(padico_psm_init);
PADICO_MODULE_FINALIZE(padico_psm_finalize);

/* ********************************************************* */

static uint64_t dummy_addr = 0; /* dummy address; real PSM addressing system uses ibverbs */

static int init_done = 0;

static void padico_psm_discover(void)
{
  int ver_major = PSM_VERNO_MAJOR;
  int ver_minor = PSM_VERNO_MINOR;
  int rc = psm_init(&ver_major, &ver_minor);
  if(rc != PSM_OK)
    {
      padico_warning("PSM: cannot init PSM library.\n");
      return;
    }
  init_done = 1;
  uint32_t num_units = -1;
  rc = psm_ep_num_devunits(&num_units);
  if(rc == PSM_OK)
    {
      padico_print("PSM: detected %d devices.\n", num_units);
      const char*s_uuid = padico_getenv("PADICO_BOOT_UUID");
      padico_string_t s_net = padico_string_new();
      padico_string_printf(s_net, "PSM-[%s]", s_uuid);
      padico_topo_network_t network = padico_na_network_getbyid(padico_string_get(s_net));
      if(network == NULL)
	{
	  network = padico_topo_network_create(padico_string_get(s_net), PADICO_NA_SCOPE_HOST, sizeof(uint64_t), AF_UNSPEC);
	}
      padico_string_delete(s_net);
      padico_na_host_bind(padico_na_getlocalhost(), network, &dummy_addr);
    }
  else
    {
      padico_print("PSM: no device found.\n");
    }
}

static int padico_psm_init(void)
{
  padico_psm_discover();
  return 0;
}

static void padico_psm_finalize(void)
{
  if(init_done)
    {
      init_done = 0;
      psm_finalize();
    }
}
