#include <Padico/Puk.h>
#include <Padico/PadicoTM.h>
#include <Padico/Module.h>
#include <Padico/Topology.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

PADICO_MODULE_DECLARE(Resolve);
PADICO_MODULE_INIT(padico_resolve_init);

static int isaddr(const char *hostname)
{
  if(hostname)
    {
      struct sockaddr_in inaddr;
      if (inet_pton(AF_INET, hostname, &inaddr)==1)
	return 1;
      struct sockaddr_in6 in6addr;
      if (inet_pton(AF_INET6, hostname, &in6addr)==1)
	return 1;
      return 0;
    }
  return -1;
}

static char* nametoipv4(const char * name)
{
  char* ipstr= padico_malloc(INET_ADDRSTRLEN);
  padico_na_host_t h=padico_na_host_resolvebyname(name);
  const struct in_addr* iphost = padico_na_host_getaddr(h);
  if(iphost)
    inet_ntop(AF_INET, iphost, ipstr, INET_ADDRSTRLEN);
  else 
    {
      padico_free(ipstr);
      ipstr = NULL;
    }
  return ipstr;
}

int padico_getaddrinfo(const char *node, const char *service,
		       const struct addrinfo *hints, struct addrinfo **res)
{
  int rc =  isaddr(node);
  if(rc == 0)
    {
      char* ipstr = nametoipv4(node);
      if(ipstr)
	{
	  rc =  PUK_ABI_RESOLV_WRAP(getaddrinfo)(ipstr, service, hints, res);
	  padico_free(ipstr);
	  return rc;
	}
    }
  return  PUK_ABI_RESOLV_WRAP(getaddrinfo)(node, service, hints, res);
}

struct hostent *padico_gethostbyname(const char *name)
{
  int rc =  isaddr(name);
  if(rc == 0)
    {
      char* ipstr = nametoipv4(name);
      if(ipstr)
	{
	  struct hostent* tmp = PUK_ABI_RESOLV_WRAP(gethostbyname)(ipstr);
	  padico_free(ipstr);
	  return tmp;
	}
    }
  return PUK_ABI_RESOLV_WRAP(gethostbyname)(name);
}

int padico_gethostbyname_r(const char *name,
			   struct hostent *ret, char *buf, size_t buflen,
			   struct hostent **result, int *h_errnop)
{
  int rc =  isaddr(name);
  if(rc == 0)
    {
      char* ipstr = nametoipv4(name);
      if(ipstr)
	{
	  rc = PUK_ABI_RESOLV_WRAP(gethostbyname_r)(ipstr, ret, buf, buflen, result, h_errnop);
	  padico_free(ipstr);
	  return rc;
	}
    }
  return PUK_ABI_RESOLV_WRAP(gethostbyname_r)(name, ret, buf, buflen, result, h_errnop);
}

int padico_gethostbyname2_r(const char *name, int af,
			    struct hostent *ret, char *buf, size_t buflen,
			    struct hostent **result, int *h_errnop)
{
  int rc =  isaddr(name);
  if(rc == 0 && af == AF_INET)
    {
      char* ipstr = nametoipv4(name);
      if(ipstr)
	{
	  rc = PUK_ABI_RESOLV_WRAP(gethostbyname2_r)(ipstr, af, ret, buf, buflen, result, h_errnop);
	  padico_free(ipstr);
	  return rc;
	}
    }
  return PUK_ABI_RESOLV_WRAP(gethostbyname2_r)(name, af, ret, buf, buflen, result, h_errnop);
}


int padico_resolve_init(void)
{

  puk_component_declare("Resolve");

#ifdef PUKABI_ENABLE_RESOLV
  puk_abi_set_virtual(getaddrinfo, &padico_getaddrinfo);
  puk_abi_set_virtual(gethostbyname, &padico_gethostbyname);
  puk_abi_set_virtual(gethostbyname_r, &padico_gethostbyname_r);
  puk_abi_set_virtual(gethostbyname2_r, &padico_gethostbyname2_r);
#endif /* PUKABI_ENABLE_RESOLV */
  
  return 0;
}
