/* @file Timing.h
 * @brief Timing service
 * @note includes a lot of copy/paste from PM2 tbx_tick
 * @author Alexandre Denis and Raymond Namyst
 */

#ifndef PADICO_TIMING_H
#define PADICO_TIMING_H


#include <sys/types.h>
#include <sys/time.h>

#include <Padico/Puk.h>

#ifdef __cplusplus
extern "C" {
#endif

#define PADICO_TIMING_GET_TICK(t)       padico_timing_get_tick(&(t))
#define PADICO_TIMING_TICK_DIFF(t1, t2) padico_timing_diff_tick(&(t1), &(t2))


  /* *** Profiling ***************************************** */

  struct padico_timing_profiler_s
  {
    int num_points;         /**< number of profiling points per iteration */
    int num_iter;           /**< number of iterations beeing profiled */
    int current_iter;       /**< current iteration number */
    padico_timing_t*points; /**< profiling points data (size = num_iter x num_points array) */
    const char*filename;    /**< filename where to flush profiling data */
    const char**labels;     /**< labels of points (size = num_points) */
  };
  typedef struct padico_timing_profiler_s* padico_timing_profiler_t;

  extern padico_timing_profiler_t padico_timing_profiler_new(int num_points,
							     int num_iter,
							     int ignore,
							     const char*filename);

  extern void padico_timing_profile_point_label(padico_timing_profiler_t p, int n_point, const char*label);

#define padico_timing_profile_point(P, N) padico_timing_profile_point_label((P), (N), __FUNCTION__)

  extern void padico_timing_profile_start(padico_timing_profiler_t p);

  extern void padico_timing_profile_flush(padico_timing_profiler_t p);

#undef PADICO_TIMING_DO_FLUSH

#ifdef __cplusplus
} /* extern "C" */
#endif


#endif /* PADICO_TIMING_H */
