/** @file 
 *  @brief Various checksum algorithms.
 */


#ifndef PUK_CHECKSUM_H
#define PUK_CHECKSUM_H

#include <stdint.h>
#ifdef __SSE2__
#include <emmintrin.h>
#endif

typedef uint32_t (*puk_checksum_func_t)(const void*_data, size_t _len);
typedef uint32_t (*puk_checksum_copy_func_t)(void*_dest, const void*_src, size_t _len);

/** a checksum algorithm */
struct puk_checksum_s
{
  const char*short_name;
  const char*name;
  puk_checksum_func_t func;
  puk_checksum_copy_func_t checksum_and_copy;
};
typedef const struct puk_checksum_s*puk_checksum_t;

extern puk_checksum_t puk_checksum_get(const char*short_name);

extern uint32_t puk_checksum_softcrc(const void*data, size_t len);


#endif /* PUK_CHECKSUM_H */

