#! /usr/bin/env -S Rscript --no-save

library(ggplot2)
library(tidyr)
library(stringr)
suppressWarnings(suppressMessages(library(dplyr)))
library(scales)


time_label <- function(x_f){
  labels <- ""
  for( i in x_f){
  const_val=c("µs","ms","s")
  x <- as.numeric(as.character(i))
  e=0
  while (x >= 10)
  {
    x=x/10
    e=e+1
  }
  m <- (e%%3)
  u <- e%/%3
  labels <- paste(labels,round(x*10^m,1),const_val[u+1]," ",sep="")
  }
  return (unlist(strsplit(labels, split=" "))) 
}

size_label <- function(x_f){
  labels <- ""
  for( i in x_f){
  const_val=c("o","ko","Mo","Go")
  x <- as.numeric(as.character(i))
  e=0
  while (x >= 10)
  {
    x=x/10
    e=e+1
  }
  m <- (e%%3)
  u <- e%/%3
  labels <- paste(labels,round(x*10^m,1),const_val[u+1]," ",sep="")
  }
  return (unlist(strsplit(labels, split=" "))) 
}

args <- commandArgs(trailingOnly=TRUE)
ref_tab <- lapply(args,read.table,header=FALSE,fill=TRUE,nrows=12)
df_tab <- lapply(args,read.table,header=FALSE,fill=TRUE,skip=44)
i=1
for(temp in df_tab){
df = data.frame(temp)
df2 = data.frame(ref_tab[i])
print(df2)
df$temp_comm = as.vector(sapply(as.vector(df$V1[seq(1, length(df$V1), 13)]), function (x) rep(x,13)))
df <- df[-seq(1, length(df$V1), 13),]
df <- extract(df,temp_comm, c("data_size","dt_time","dr_time"), "\\[([[:digit:]]+)-([[:digit:]]+)-([[:digit:]]+\\.[[:digit:]]+)\\]", convert=TRUE)
df <- extract(df,V1, c("mt_time","matrix_size","mr_time"), "\\[([[:digit:]]+)-([[:digit:]]+)-([[:digit:]]+\\.[[:digit:]]+)\\]", convert=TRUE)
  colnames(df) <- c("mt_time","matrix_size","mr_time","tcall_min","tcalc_min","twait_min","ttotal_min","tcall_max","tcalc_max","twait_max","ttotal_max","tcall","tcalc","twait","ttotal","overhead","data_size","dt_time","dr_time")
df$dr_time_min = rep(df2$V3, each =12)
df$dr_time_max = rep(df2$V4, each =12)
df$dr_time_med = rep(df2$V5, each =12)
df$dr_time = df$dr_time_max
df$oh_min<-round((df$ttotal_min - pmax(df$dr_time,df$mr_time))/pmin(df$dr_time,df$mr_time),2)
df$oh_max<-round((df$ttotal_max - pmax(df$dr_time,df$mr_time))/pmin(df$dr_time,df$mr_time),2)
df$oh_med<-round((df$ttotal - pmax(df$dr_time,df$mr_time))/pmin(df$dr_time,df$mr_time),2)
df$prop_wait = round(df$twait/df$ttotal*100,2)
df$prop_calc = round(df$tcalc/df$ttotal*100,2)
df$prop_call = round(df$tcall/df$ttotal*100,2)
df$prop_wait_min = round(df$twait_min/df$ttotal_min*100,2)
df$prop_calc_min = round(df$tcalc_min/df$ttotal_min*100,2)
df$prop_call_min = round(df$tcall_min/df$ttotal_min*100,2)
df$prop_wait_max = round(df$twait_max/df$ttotal_max*100,2)
df$prop_calc_max = round(df$tcalc_max/df$ttotal_max*100,2)
df$prop_call_max = round(df$tcall_max/df$ttotal_max*100,2)
df$matrix_size <- as.factor(df$matrix_size)
df$dt_time <- as.factor(df$dt_time)
df$mt_time <- as.factor(df$mt_time)
df$dr_time <- as.factor(df$dr_time)
df$mr_time <- as.factor(df$mr_time)


name <- str_remove_all(args[i],".dat")

case <- filter(df, dt_time == mt_time)#, mt_time == 1024000)
oh <- squish(case$oh_med,c(-1,2))
oh_min <-c(squish(case$oh_min,c(-1,2)),rep(c(0),length(case$prop_wait)*2))
oh_max <-c(squish(case$oh_max,c(-1,2)),rep(c(0),length(case$prop_wait)*2))
prop_wt <- case$prop_wait/100
prop_cc <- case$prop_calc/100
prop_cl <- case$prop_call/100

xbar <-  rep(1:length(case$prop_wait), 3)
ybar <- c(oh*prop_wt,oh*prop_cc,oh*prop_cl)
yminbar <- c(oh_min)
ymaxbar <- c(oh_max)
cond <- as.factor(rep(c(1,2,3),each=length(case$prop_wait)))
df2 <-data.frame(xbar,ybar,yminbar,ymaxbar,cond)

title_name <- basename(name)

ggplot(data=df2, aes(x=xbar, y=ybar, fill=cond)) + 
  geom_col() + 
  labs(title = paste("Precise overview:",title_name)) + 
  scale_x_discrete("[collective time, compute_time]",labels=paste("[",time_label(case$dt_time),",",time_label(case$mt_time),"]",sep=""),breaks=unique(xbar),limits=unique(xbar)) + scale_y_continuous(name="ratio", breaks = c(-1,0,1,2),limits=c(-1,2),oob=scales::squish) + theme(axis.text.x=element_text(angle=0))+ geom_errorbar(ymin=yminbar, ymax=ymaxbar) + geom_hline(yintercept=0,color = "green",size=0.1) + 
  geom_hline(yintercept=1,color = "black",size=1)+ 
  geom_hline(yintercept=2,color = "red",size=0.1)+ 
  geom_hline(yintercept=-1,color = "blue",size=0.1)+
  theme(axis.text.x = element_text(angle = 20, vjust = 1, hjust=0.8))+
  scale_fill_manual("proportion", values=c("1" = "darkorchid3","2" = "royalblue4", "3" = "turquoise3"), labels=c("MPI_Wait","Compute","MPI Call"))
i=i+1
  print (paste(name,"_precise.pdf"))
ggsave(paste(name,"_precise.pdf",sep=""), width = 7, height = 4, dpi=1200)
}
