#! /usr/bin/env -S Rscript --no-save

library(ggplot2)
library(tidyr)
library(stringr)
suppressWarnings(suppressMessages(library(dplyr)))
library(scales)

INTERPOLATE<-FALSE
BIG_LEGEND<-TRUE
BOUND<-10000
time_label <- function(x_f){
  labels <- ""
  for( i in x_f){
  const_val=c("µs","ms","s")
  x <- as.numeric(as.character(i))
  e=0
  while (x >= 10)
  {
    x=x/10
    e=e+1
  }
  m <- (e%%3)
  u <- e%/%3
  labels <- paste(labels,round(x*10^m,1),const_val[u+1]," ",sep="")
  }
  return (unlist(strsplit(labels, split=" "))) 
}

size_label <- function(x_f){
  labels <- ""
  for( i in x_f){
  const_val=c("o","ko","Mo","Go")
  x <- as.numeric(as.character(i))
  e=0
  while (x >= 10)
  {
    x=x/10
    e=e+1
  }
  m <- (e%%3)
  u <- e%/%3
  labels <- paste(labels,round(x*10^m,1),const_val[u+1]," ",sep="")
  }
  return (unlist(strsplit(labels, split=" "))) 
}

args <- commandArgs(trailingOnly=TRUE)
df_tab <- lapply(args,read.table,header=FALSE,fill=TRUE,skip=44)
implem <- gsub("^.*\\/","", args)
title_tab <- unlist(gsub("\\_raw.*","",implem))
i=1
for(temp in df_tab){
df = data.frame(temp)
df$temp_comm = as.vector(sapply(as.vector(df$V1[seq(1, length(df$V1), 13)]), function (x) rep(x,13)))
df <- df[-seq(1, length(df$V1), 13),]
df <- extract(df,temp_comm, c("data_size","dt_time","dr_time"), "\\[([[:digit:]]+)-([[:digit:]]+)-([[:digit:]]+\\.[[:digit:]]+)\\]", convert=TRUE)
df <- extract(df,V1, c("mt_time","matrix_size","mr_time"), "\\[([[:digit:]]+)-([[:digit:]]+)-([[:digit:]]+\\.[[:digit:]]+)\\]", convert=TRUE)
  colnames(df) <- c("mt_time","matrix_size","mr_time","tcall_min","tcalc_min","twait_min","ttotal_min","tcall_max","tcalc_max","twait_max","ttotal_max","tcall","tcalc","twait","ttotal","overhead","data_size","dt_time","dr_time")
df$prop_calc = round(((df$tcalc-df$mr_time)+(df$twait+df$tcall-df$dr_time)),2)
df$prop_calc <- squish(df$prop_calc,c(-BOUND,BOUND))
df$matrix_size <- as.factor(df$matrix_size)
df$dt_time <- as.factor(df$dt_time)
df$mt_time <- as.factor(df$mt_time)
df$dr_time <- as.factor(df$dr_time)
df$mr_time <- as.factor(df$mr_time)
uds <- unique(as.vector(df[c("data_size", "dt_time")]$data_size))
if(BIG_LEGEND)
{ 
  lblsize <- 18
  labelx <-paste(time_label(as.vector(unique(df$dt_time))),sep="\n") 
  labely <- paste(time_label(as.vector(unique(df$mt_time))),sep="\n")
  xname <- "Collective time"
  sub = NULL
}else{
  xname <- "Collective size and times"
  lblsize <- 10
  sub <- paste("File: ",args[i],sep="")
labelx <- paste(size_label(as.vector(uds)),time_label(as.vector(unique(df$dt_time))),time_label(as.vector(unique(df$dr_time))),sep="\n")
labely <- paste(unlist(as.vector(unique(df$matrix_size))),time_label(as.vector(unique(df$mt_time))),time_label(as.vector(unique(df$mr_time))),sep="\n")
}
g <- ggplot(df, aes(x=dr_time, y=mr_time))+
  labs(title = title_tab[i], subtitle=sub)+
  theme(plot.title = element_text(hjust = 0.5))+
  theme(plot.subtitle = element_text(hjust = 0.5))+
	geom_raster(aes(fill=df$prop_calc),interpolate=INTERPOLATE)+
  theme(panel.background=element_rect(fill="transparent",colour=NA))+
  theme(panel.grid.major = element_line(colour = "black",size=0.01))+
  scale_x_discrete(name=xname,breaks=unique(df$dr_time), labels=labelx)+
  scale_y_discrete(name="Matrix compute time",breaks=unique(df$mr_time), labels=labely)+
  #scale_fill_gradientn(colours=c("dodgerblue3","purple"),oob=scales::squish,breaks=c(0,100),labels=c("0","100"),values=c(0,100),minor_breaks=NULL,space = "Lab",na.value = "#00000000", guide = guide_colourbar(direction="vertical"), aesthetics = "fill",limits=c(0,100))+
  scale_fill_gradientn(colours=c("royalblue4","darkorchid3","firebrick"),name="gen. ratio",aesthetics = "fill",limits=c(-BOUND,BOUND),breaks=c(-BOUND,0,BOUND),labels=c("0%","100%","200%"))+
  
  theme(legend.position="right",title=element_text(size = 20),axis.text=element_text(size = lblsize),axis.text.x=element_text(angle=-90), axis.title=element_text(size = 20))
name <- str_remove(args[i],".dat")
file2 <-paste(name,"_all.pdf",sep="")
i=i+1
ggsave(file2, width = 9, height = 8, dpi = 1200)
}
