/*
 * PM2: Parallel Multithreaded Machine
 * Copyright (C) 2001 the PM2 team (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */


#ifndef __INLINEFUNCTIONS_ASM_X86_64_LINUX_SYSTEM_H__
#define __INLINEFUNCTIONS_ASM_X86_64_LINUX_SYSTEM_H__


/*
 * System defines. Note that this is included both from .c and .S
 * files, so it does only defines, not any C code.  This is based
 * on information published in the Processor Abstraction Layer
 * and the System Abstraction Layer manual.
 *
 * Copyright (C) 1998-2003 Hewlett-Packard Co
 *	David Mosberger-Tang <davidm@hpl.hp.com>
 * Copyright (C) 1999 Asit Mallick <asit.k.mallick@intel.com>
 * Copyright (C) 1999 Don Dugger <don.dugger@intel.com>
 */


#include "asm/linux_system.h"


#ifdef __MARCEL_KERNEL__
TBX_VISIBILITY_PUSH_INTERNAL


/** Internal functions **/
/*
 * Note: no "lock" prefix even on SMP: xchg always implies lock anyway
 * Note 2: xchg has side effect, so that attribute volatile is necessary,
 *	  but generally the primitive is invalid, *ptr is output argument. --ANK
 */
static inline unsigned long __ma_xchg(unsigned long x, volatile void *ptr, int size)
{
	switch (size) {
	case 1:
	      __asm__ __volatile__("xchgb %b0,%1":"=q"(x)
	      :		     "m"(*__ma_xg(ptr)), "0"(x)
	      :		     "memory");
		break;
	case 2:
	      __asm__ __volatile__("xchgw %w0,%1":"=r"(x)
	      :		     "m"(*__ma_xg(ptr)), "0"(x)
	      :		     "memory");
		break;
	case 4:
	      __asm__ __volatile__("xchgl %k0,%1":"=r"(x)
	      :		     "m"(*__ma_xg(ptr)), "0"(x)
	      :		     "memory");
		break;
	case 8:
	      __asm__ __volatile__("xchgq %0,%1":"=r"(x)
	      :		     "m"(*__ma_xg(ptr)), "0"(x)
	      :		     "memory");
		break;
	default:
		abort();
	}
	return x;
}

/*
 * Atomic compare and exchange.  Compare OLD with MEM, if identical,
 * store NEW in MEM.  Return the initial value in MEM.  Success is
 * indicated by comparing RETURN with OLD.
 */
static __tbx_inline__ unsigned long TBX_NOINST __ma_cmpxchg(volatile void *ptr,
							    unsigned long old,
							    unsigned long repl, int size)
{
	unsigned long prev;
	switch (size) {
	case 1:
	      __asm__ __volatile__(MA_LOCK_PREFIX "cmpxchgb %b1,%2":"=a"(prev)
	      :		     "q"(repl), "m"(*__ma_xg(ptr)),
				     "0"(old)
	      :		     "memory");
		return prev;
	case 2:
	      __asm__ __volatile__(MA_LOCK_PREFIX "cmpxchgw %w1,%2":"=a"(prev)
	      :		     "q"(repl), "m"(*__ma_xg(ptr)),
				     "0"(old)
	      :		     "memory");
		return prev;
	case 4:
	      __asm__ __volatile__(MA_LOCK_PREFIX "cmpxchgl %k1,%2":"=a"(prev)
	      :		     "q"(repl), "m"(*__ma_xg(ptr)),
				     "0"(old)
	      :		     "memory");
		return prev;
	case 8:
	      __asm__ __volatile__(MA_LOCK_PREFIX "cmpxchgq %1,%2":"=a"(prev)
	      :		     "q"(repl), "m"(*__ma_xg(ptr)),
				     "0"(old)
	      :		     "memory");
		return prev;
	default:
		abort();
	}
	return old;
}


TBX_VISIBILITY_POP
#endif /** __MARCEL_KERNEL__ **/


#endif /** __INLINEFUNCTIONS_ASM_X86_64_LINUX_SYSTEM_H__ **/
