\page README README

NewMadeleine README
===================

This document describes nmad installation and configuration.

for any question, mailto: Alexandre.Denis@inria.fr

Prerequisites
-------------

The following development tools are required to compile NewMadeleine:
  - GNU C Compiler `gcc` (version 3.2 and higher) or compatible (`icc`, `clang`).
  - GNU `make` (version 3.81 and higher).
  - autoconf (v 2.50 or later)
  - pkg-config
  - hwloc (optional, recommended)
  - libexpat XML parser
  - ibverbs/OFED for IB support (set $IBHOME if not installed in /usr)
  - PSM for Intel InfiniPath
  - PSM2 for Intel OmniPath
  - MX for Myrinet support (set $MX_DIR if not installed in /usr)
  - libpmi2 for PMI2 (slurm) integration
  - other PM2 modules:
    * Puk: mandatory
    * PadicoTM: recommended, used as launcher
    * pioman: optionnal, for progression
    * these modules are built automatically when using the automated 
      build system.

Installation
------------

Installation of NewMadeleine is a standard sequence of `./configure ; make ; make install`.
However, to build and install all modules required by NewMadeleine, we propose
a script to build the modules with the right order and parameters.

**Automated build (recommended)**:

To build all modules required by nmad, we recommend to use the build
script located in pm2/scripts/pm2-build-packages using a given or a
custom configuration file, e.g.:

     % cd pm2/scripts
     % ./pm2-build-packages ./madmpi.conf --prefix=$HOME/soft/x86_64

For a standard multi-threaded build, it is advised to use `madmpi.conf`.
For a non-threaded (no progression!), configuration from `madmpi-mini.conf`
will lead to a slightly more efficient library.

**Manual build (not recommended, advanced users only)**.

Module nmad requires other pm2 modules: Puk, PadicoTM, PukABI (optionnal),
pioman (optionnal).

For each module:

    ./autogen.sh
    mkdir build ; cd build
    ../configure [your options here]
    make
    make install

*Note*: nmad purposely cannot be configured in its source
 directory. Please use a separate build directory.

Usefull configure flags (see ./configure --help)

    --enable-sampling       Enable network sampling
    --enable-nuioa          Enable NUMA control on I/O
    --enable-mpi            Enable builtin MPI implementation MadMPI
    --with-pioman           use pioman I/O manager [default=no]
    --with-ibverbs          use Infiniband ibverbs [default=check]
    --with-mx               use Myrinet MX [default=check]
    --with-psm              use Intel Performance Scaled Messasging (PSM) [default=check]
    --with-psm2             use Intel Performance Scaled Messasging 2 (PSM2) [default=check]


Building application code
-------------------------

For an MPI applicatiion using MadMPI, use the standard `mpicc`,
`mpif77` and `mpif90` compiler frontends to build and link.

To build an application using native NewMadeleine interface, get the
required flags through pkg-config. For CFLAGS:

    % pkg-config --cflags nmad

For libraries:

    % pkg-config --libs nmad

In a `Makefile`, you will typically need:

    CFLAGS += $(shell pkg-config --cflags)
    LIBS += $(shell pkg-config --libs)


Launcher
--------

For MadMPI use the standard `mpirun` as launcher. Please see `mpirun --help`
for up-to-date documentation. Please note that MadMPI `mpirun` is a
frontend to `padico-launch` so it accepts all options described below.

For native NewMadeleine applications, it is recommended to use
`padico-launch` as a launcher for nmad. It accepts parameters similar
to `mpirun`. Please see `padico-launch --help` for up-to-date
documentation. For example:

    % padico-launch -n 2 -nodelist jack0,jack1 nm_bench_sendrecv

starts program 'nm_bench_sendrecv' on hosts jack0 and jack1, using
auto-detected network.

Environment variables may be set using -D parameters, e.g.:

    % padico-launch -c -p -n 2 -nodelist jack0,jack1 -DNMAD_DRIVER=ib nm_bench_sendrecv

starts program 'nm_bench_sendrecv' on hosts jack0 and jack1, over
Infiniband, using one console per process.

On clusters using slurm, `mpirun` and `padico-launch` will start processes
using `srun`. It is possible for the user to directly use `srun` without using
`mpirun` at all. Make sure to use pmi2, e.g.:

    % srun -N 2 --exclusive --mpi=pmi2 nm_bench_sendrecv

Debug
-----

To launch each process in a `gdb` debugger, use argument `-d` in
association with `-c` to get one console per node, i.e.:

    % padico-launch -c -d -n 2 nm_sr_hello

To launch each process in `valgrind` memcheck tool, use argument
`--padico-valgrind`, in association with `-c`, i.e.:

    % padico-launch -c --padico-valgrind -n 2 nm_sr_hello

NewMadeleine comes with a tool to detect if user data is modified
while a non-blocking send is manipulating the data, which leads to
data corruption. To detect such a bug in application code, please set
the environment variable `NMAD_ISEND_CHECK` to a non-null value, i.e.:

    % padico-launch -DNMAD_ISEND_CHECK=1 nm_sr_hello

It checks whether user buffer has been modified between `nm_sr_isend`
and `nm_sr_swait`, or between `MPI_Isend` and `MPI_Test` or
`MPI_Wait`. This feature is only available when nmad is built in debug
mode.


Advanced Tuning
---------------

### Strategy

The strategy used by nmad is selected using the following rules:

1. if the environment variable `NMAD_STRATEGY` is set, it is used
whatever the other configuration parameters are.

2. if the variable is not set, strategy 'aggreg' is used by default.

Valid strategies are: 
  default, aggreg, aggreg_autoextended, split_balance, prio.

The following are deprecated/unmatained:
  split_all, qos

The default choice should fit most cases.

### Drivers

The drivers used by nmad are selected using the following rules:

1. if the environment variable NMAD_DRIVER is set, it is used by
default. It may contain the name of a single driver for single rail,
or a list of multiple drivers separated by '+' for multirail,
e.g. NMAD_DRIVER=mx+ibverbs
An optionnal board number may be given if multiple boards with the
same driver are present, e.g. NMAD_DRIVER=ibverbs:0+ibverbs:1 for
dual-IB.

2. if nmad is launched with `mpirun` or `padico-launch`, then PadicoTM
default NetSelector rules apply:
  + intra-process uses 'self'
  + intra-node inter-process uses 'shm'
  + inter-node uses 'psm2' if OmniPath is present (auto-detected)
  + inter-node uses 'psm' if InfiniPath is present (auto-detected)
  + inter-node uses 'ibverbs' if InfiniBand is present (auto-detected)
    and nodes are on the same IB subnet (same subnet manager). Thus it
    is important to configure subnet GID prefix and not keep the
    factory default GID prefix in `opensm`.
  + inter-node uses 'mx' if Myrinet is auto-detetected and nodes are
    on the same Myrinet network (same mapper).
  + inter-node uses 'tcp' if IP is available, and neither IB nor MX
    are available.
  + as last resort, routed messages over control channel is used if
    no direct connection is possible.
Usual NetSelector customization is possible.

3. if nmad is launched through the cmdline launcher, then a
"-R <string>" parameter is taken as a railstring, with the same syntax
as NMAD_DRIVER. Please note that cmdline launcher is only for debug
purpose and manages only 2 nodes.

4. if another custom launcher is used, it may set a selector using the
'sesion' interface.

5. in any other case, 'self' is used for intra-process; 'tcp' for
inter-process.


Strategy 'prio' limits the total number of simultaneous outgoing
packets. This number may be tuned using environment variable
NM_PRIO_MAX_PW. The dfault value is 2.


### General tuning

- NMAD_AUTO_FLUSH=1 asks nmad to flush outgoing packets after every
posted send. It ensures data is sent earlier; in return, it increses
contention between threads and prevent the 'aggreg' strategy to
actually aggregate messages.


### Debug

- NMAD_ISEND_CHECK=1 checksum data between the isend and communication
  completion, to check whether user touched data when he was not
  supposed to.


### Infiniband tuning

Infiniband may be tuned at run time through environment variables:
- NMAD_IBVERBS_RCACHE=1 enables the registration cache; the default
  choice is to use the rcache-mini backend, best used with the PukABI
  module. Other backend, bringing other mechanisms to maintain
  registration cache consistency may be selected with the following
  variables:
    - NMAD_RCACHE_ODP=1 memory blocks are registered for On-Demand
      Paging (ODP), with page fault prefetch, without cache. Performance
      is poor.
    - NMAD_RCACHE_IODP=1 the full process memory is registered for
      Implicit On-Demand Paging (IODP), with prefetch. This mode is
      available on ConnectX-5+ hardware. It is safe to use, compatible
      with any memory allocator, and gets fair performance (although
      a little slower than default choice).
    - NMAD_RCACHE_NOCACHE=1 no cache is implemented, memory is registered
      and deregistered for each packet. It is used for debugging purpose
      only; end-users are not expected to use it.
- NMAD_IBVERBS_SRQ=1 enables the use of Shared Requests Queues for
  scalability in number of nodes. The performance penalty is low.
- NMAD_IBVERBS_CHECKSUM=1 enables the checksum computation on the fly
   in the driver.
- NMAD_IBVERBS_ALIGN=<n> sets alignment of every packets sent through
   InfiniBand to <n>, using padding. Default is 64 bytes.
- NMAD_IBVERBS_MEMALIGN=<n> enforces alignment of buffers used
   internally in all InfiniBand drivers. Default is 4096 bytes.
- NMAD_IBVERBS_COMP_CHANNEL=1 enables the completion channel (to use
   blocking syscalls) with driver 'srq'.
- NMAD_RCACHE_CHECKSUM=1 enables checksums for the 'rcache' driver,
for debugging purpose only.


When `opensm` is used as subnet manager, subnet GID must be customized
with a value unique to the given subnet, so as nmad is able to
automatically detect IB connectivity. As root:
- create the default opensm config file:

      % opensm -o -c /var/cache/opensm/opensm.opts

- in the above file, customize the line with subnet_prefix to some
other value than the factory default 0xfe80000000000000. Set the same
subnet GID on all nodes of the subnet.
- restart opensm:

      % /etc/init.d/infiniband restart


### Launcher advanced tuning

The appropriate launcher to use is usually selected automatically. For
testing and debugging, it may be forced using environment variable
NM_LAUNCHER. Valid values are:
- 'madico': use PadicoTM as launcher.
- 'pmi2': use slurm PMI2
- 'cmdline': processes are launched by user, connection information if
   given on command-line. This launcher is able to launch only 2
   processes per job.
The default is 'pmi2' if a slurm job is detected with pmi2 enabled,
'madico' if we detect job was launched with 'padico-launch', 'cmdline'
if nothing else is available.


### Multicast tuning

The default routing tree for the multicast interface is a binomial one. You can
change it with the environment variable NMAD_MCAST_TREE set to `binary`,
`binomial` or `chain`.


Documentation
-------------

To generate doxygen documentation:

    % cd $prefix/build/nmad
    % make docs

It is available online at http://pm2.gforge.inria.fr/newmadeleine/doc/

