/*
 * NewMadeleine
 * Copyright (C) 2006-2020 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <math.h>
#include <stdio.h>
#include <nm_sendrecv_interface.h>
#include <nm_coll_interface.h>

static void nm_coll_tree_display(nm_coll_tree_kind_t kind, const char*name, int root, int n, int with_weight)
{
  struct nm_coll_tree_info_s trees[n];
  int*children[n];
  printf("# %s (kind = %d); n = %d\n", name, kind, n);
  int i;
  for(i = 0; i < n; i++)
    {
      nm_coll_tree_init(&trees[i], kind, n, i, root);
      children[i] = malloc(sizeof(int) * trees[i].max_arity);
    }  
  int step;
  for(step = 0; step < trees[0].height; step++)
    {
      printf("step = %d / %d\n", step, trees[0].height);
      for(i = 0; i < n; i++)
        {
          int parent = -1;
          int n_children = 0;
          nm_coll_tree_step(&trees[i], step, &parent, children[i], &n_children);
          printf("  i = %d; parent = %d; ", i, parent);
          int k;
          for(k = 0; k < n_children;k++)
            {
              printf("child[%d] = %d; ", k, children[i][k]);
            }
           printf("weight = %d\n", with_weight ? nm_coll_tree_weight(&trees[i], step, i) : -1);
        }      
    }
  for(i = 0; i < n; i++)
    {
      free(children[i]);
    }
  printf("# %s; n = %d; done.\n\n\n", name, n);
}

int main(int argc, char**argv)
{
  const int n = 63;
  const int root = 0;

  nm_coll_tree_display(NM_COLL_TREE_BINOMIAL, "binomial tree", root, n, 1);
  nm_coll_tree_display(NM_COLL_TREE_4NOMIAL, "4-nomial tree", root, n, 0);
  nm_coll_tree_display(NM_COLL_TREE_BINARY, "binary tree", root, n, 0);
  
  return 0;
}
