/*
 * NewMadeleine
 * Copyright (C) 2021 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <stdlib.h>
#include <stdio.h>

#include <nm_launcher_interface.h>
#include <nm_coll_interface.h>
#include <nm_mcast_interface.h>


int main(int argc, char**argv)
{
  int worldsize = 0;
  int rank = -1;
  const char msg[] = "Hello, world!";
  const nm_len_t msg_size = strlen(msg) + 1;
  const nm_tag_t tag = 0x02;
  const int hlen = 0;

  /* generic nmad init */
  nm_launcher_init(&argc, argv);
  nm_launcher_get_size(&worldsize);
  nm_launcher_get_rank(&rank);
  nm_comm_t p_comm = nm_comm_world("nm_mcast_hello");
  nm_mcast_init();

  if (rank == 0)
    {
      /* send mcast */
      int* dests = malloc(sizeof(int) * (worldsize - 1));
      int i;
      for (i = 1; i < worldsize; i++)
        {
          dests[i-1] = i;
        }
      struct nm_data_s data;
      nm_data_contiguous_build(&data, (char*) msg, msg_size);
      nm_mcast_send(p_comm, dests, NULL, worldsize-1, tag, &data, hlen, NM_COLL_TREE_DEFAULT);
      free(dests);
    }
  else
    {
      /* receive data from an mcast with a point-to-point receive */
      nm_session_t p_session = nm_comm_get_session(p_comm);
      char* buffer = malloc(msg_size);
      nm_sr_request_t request;
#if 1 // detailled recv:
      nm_sr_recv_init(p_session, &request);
      nm_sr_recv_match(p_session, &request, NM_ANY_GATE, tag, NM_TAG_MASK_FULL);
      nm_sr_recv_post(p_session, &request);
      nm_sr_recv_data_wait(p_session, &request);
      struct nm_data_s data;
      nm_data_contiguous_build(&data, buffer, hlen);
      int rc = nm_sr_recv_peek(p_session, &request, &data);
      assert(rc == NM_ESUCCESS);
      nm_sr_recv_offset(p_session, &request, hlen);
      nm_sr_recv_unpack_contiguous(p_session, &request, buffer, msg_size);
#else // generic recv:
      nm_sr_irecv(p_session, 0, tag, buffer, msg_size, &request);
#endif
      nm_sr_rwait(p_session, &request);
      printf("Received message: %s\n", buffer);
      free(buffer);
    }

  /* generic nmad termination */
  nm_coll_barrier(p_comm, 0xF2);
  nm_mcast_finalize();
  nm_comm_destroy(p_comm);
  nm_launcher_exit();
  return 0;
}
