/*
 * NewMadeleine
 * Copyright (C) 2006-2019 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <nm_onesided_interface.h>
#include "../common/nm_examples_helper.h"
#include <stdio.h>

static int a = 0;

int main(int argc, char**argv)
{
  const nm_tag_t tag = 0x01;
  nm_examples_init(&argc, argv);

  /* intialize the onesided interface */
  nm_onesided_t p_onesided;
  nm_onesided_init(p_session, &p_onesided);

  if(is_server)
    {
      int*p_a = &a;
      nm_sr_send(p_session, p_gate, tag, &p_a, sizeof(int*));
      nm_coll_barrier(p_comm, tag);
      fprintf(stderr, "a = %d\n", a);
      nm_coll_barrier(p_comm, tag);
      fprintf(stderr, "a = %d\n", a);
      a = 3;
      nm_coll_barrier(p_comm, tag);
    }
  else
    {
      /* receive address for remote variable 'a' */
      int*p_a = NULL;
      nm_sr_recv(p_session, p_gate, tag, &p_a, sizeof(int*));
      a = 1;
      /* build data descriptor for local a */
      struct nm_data_s data;
      nm_data_contiguous_build(&data, &a, sizeof(a));
      /* non-blocking put */
      nm_onesided_request_t req;
      nm_onesided_iput(p_onesided, p_gate, &data, (uintptr_t)p_a, &req);
      nm_onesided_req_wait(&req);
      /* memory fence & barrier */
      nm_onesided_fence(p_onesided, p_gate);
      nm_coll_barrier(p_comm, tag);
      a = 2;
      /* blocking put */
      nm_onesided_put(p_onesided, p_gate, &data, (uintptr_t)p_a);
      /* memory fence & barrier */
      nm_onesided_fence(p_onesided, p_gate);
      nm_coll_barrier(p_comm, tag);
      /* non-blocking get */
      nm_onesided_iget(p_onesided, p_gate, &data, (uintptr_t)p_a, &req);
      nm_onesided_req_wait(&req);
      fprintf(stderr, "a = %d\n", a);
      nm_coll_barrier(p_comm, tag);
    }

  /* stop the onesided interface */
  nm_onesided_finalize(p_onesided);
  nm_examples_exit();  
}

