/*
 * NewMadeleine
 * Copyright (C) 2006-2021 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef NM_ERRNO_H
#define NM_ERRNO_H

/** @ingroup nm_public 
 * @{
 * @file
 * This file contains error codes definitions.
 * @}
 */

/** @ingroup core_interface
 * @{
 */

/** Error codes.
 */
enum nm_errno_e
  {
    NM_ESUCCESS	        =  0,	/**< successful operation */
    NM_EUNKNOWN	        =  1,	/**< unknown error */
    NM_ENOTIMPL	        =  2,	/**< not implemented */
    NM_ESCFAILD	        =  3,	/**< syscall failed, see errno */
    NM_EAGAIN	        =  4,	/**< poll again */
    NM_ECLOSED	        =  5,	/**< connection closed */
    NM_EBROKEN	        =  6,	/**< error condition on	connection */
    NM_EINVAL	        =  7,	/**< invalid parameter */
    NM_ENOTFOUND        =  8,	/**< not found */
    NM_ENOMEM	        =  9,	/**< out of memory */
    NM_EALREADY	        = 10,	/**< already in progress or done */
    NM_ETIMEDOUT        = 11,	/**< operation timeout */
    NM_EINPROGRESS	= 12,	/**< operation in progress */
    NM_EUNREACH	        = 13,	/**< destination unreachable */
    NM_ECANCELED	= 14,	/**< operation canceled */
    NM_EABORTED		= 15,	/**< operation aborted */
    NM_EBUSY            = 16,   /**< gate/track is busy */
    NM_ENOTPOSTED       = 17,   /**< request not posted */
    NM_ETRUNCATED       = 18,   /**< receive is too short, message would be truncated */
    _NM_ERR_MAX
  };

static inline const char*nm_strerror(int rc)
{
  static const char*error_msg[_NM_ERR_MAX] =
    {
     [NM_ESUCCESS]    = "success",
     [NM_EUNKNOWN]    = "unknown error",
     [NM_ENOTIMPL]    = "not implemented",
     [NM_ESCFAILD]    = "syscall failed",
     [NM_EAGAIN]      = "poll again",
     [NM_ECLOSED]     = "connection closed",
     [NM_EBROKEN]     = "error condition on connection",
     [NM_EINVAL]      = "invalid parameter",
     [NM_ENOTFOUND]   = "not found",
     [NM_ENOMEM]      = "out of memory",
     [NM_EALREADY]    = "already in progress",
     [NM_ETIMEDOUT]   = "operation timeout",
     [NM_EINPROGRESS] = "operation in progress",
     [NM_EUNREACH]    = "destination unreachable",
     [NM_ECANCELED]   = "operation canceled",
     [NM_EABORTED]    = "operation aborted",
     [NM_EBUSY]       = "gate/trk is busy",
     [NM_ENOTPOSTED]  = "request not posted",
     [NM_ETRUNCATED]  = "message truncated"
    };
  if(rc <= 0 && rc > -_NM_ERR_MAX)
    {
      return error_msg[-rc];
    }
  else
    {
      return "out of bound error code.";
    }
}

/** @}
 */

#endif /* NM_ERRNO_H */
