/*
 * NewMadeleine
 * Copyright (C) 2006-2021 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */


#ifndef NM_PRIVATE_H
#define NM_PRIVATE_H

#include "nm_private_config.h"
#include <Padico/Puk.h>
#ifdef PIOMAN
#include <pioman.h>
#endif

#if defined(PUKABI)
#include <Padico/Puk-ABI.h>
#endif /* PUKABI */
#if defined(PUKABI_ENABLE_FSYS)
#define NM_SYS(SYMBOL) PUK_ABI_FSYS_WRAP(SYMBOL)
#else  /* PUKABI_ENABLE_FSYS */
#define NM_SYS(SYMBOL) SYMBOL
#endif /* PUKABI_ENABLE_FSYS */

#ifndef NMAD_BUILD
#  error "NMAD_BUILD flag not set while including nm_private.h. This header is not part of API and cannot be used outside of nmad."
#endif /* NMAD_BUILD */

#include <nm_public.h>
#include <nm_core_interface.h>
#include <nm_log.h>

#include <nm_parameters.h>
#include <nm_trace.h>

#include <Padico/Module.h>

#ifdef NMAD_HWLOC
#include <hwloc.h>
#endif

#include <nm_core_private.h>
#include <nm_tags.h>
#include <nm_pkt_wrap.h>
#include <nm_minidriver.h>
#include <nm_drv.h>
#include <nm_log.h>
#ifdef PIOMAN
#include <nm_piom_ltasks.h>
#endif
#include <nm_headers.h>
#include <nm_strategy.h>
#include <nm_gate.h>
#include <nm_core.h>
#include <nm_lock.h>

__PUK_SYM_INTERNAL
void nm_core_driver_flush(struct nm_core *p_core);

__PUK_SYM_INTERNAL
void nm_core_driver_exit(struct nm_core *p_core);

__PUK_SYM_INTERNAL
void nm_core_gate_exit(struct nm_gate_s*p_gate);

__PUK_SYM_INTERNAL
void nm_core_status_event(nm_core_t p_core, const struct nm_core_event_s*const p_event, struct nm_req_s*p_req);

__PUK_SYM_INTERNAL
void nm_core_events_dispatch(struct nm_core*p_core);

__PUK_SYM_INTERNAL
void nm_unexpected_clean(struct nm_core*p_core);

__PUK_SYM_INTERNAL
void nm_core_progress(struct nm_core*p_core);

__PUK_SYM_INTERNAL
void nm_core_task_flush(struct nm_core*p_core);

__PUK_SYM_INTERNAL
void nm_drv_refill_recv(nm_drv_t p_drv, nm_gate_t p_gate);

__PUK_SYM_INTERNAL
void nm_gate_disconnected(struct nm_core*p_core, nm_gate_t p_gate, nm_drv_t p_drv);

void nm_pw_post_send(struct nm_pkt_wrap_s*p_pw);

void nm_pw_poll_send(struct nm_pkt_wrap_s *p_pw);

void nm_pw_recv_progress(struct nm_pkt_wrap_s*p_pw);

__PUK_SYM_INTERNAL
void nm_pw_recv_wait(struct nm_pkt_wrap_s*p_pw);

__PUK_SYM_INTERNAL
void nm_out_prefetch(struct nm_core*p_core);

/** Process a complete successful outgoing request.
 */
void nm_pw_process_complete_send(struct nm_core*p_core, struct nm_pkt_wrap_s*p_pw);

/** Process complete incoming request.
 */
void nm_pw_process_complete_recv(struct nm_core*p_core, struct nm_pkt_wrap_s*p_pw);

void nm_data_pkt_pack(struct nm_pkt_wrap_s*p_pw, nm_core_tag_t tag, nm_seq_t seq,
		      const struct nm_data_s*p_data, nm_len_t chunk_offset, nm_len_t chunk_len, uint8_t flags);

void nm_data_pkt_unpack(const struct nm_data_s*p_data, const struct nm_header_pkt_data_s*h, const struct nm_pkt_wrap_s*p_pw,
			nm_len_t chunk_offset, nm_len_t chunk_len);

void nm_core_unpack_trigger_next(struct nm_core*p_core, struct nm_unpack_next_s*p_next);

void nm_core_matching_check(struct nm_core*p_core);

#include "nm_core_inline.h"
#include "nm_tactics.h"
#include "nm_sampling.h"

#endif /* NM_PRIVATE_H */
