#include "nm_connector.h"
#include <nm_private_config.h>
#include <nm_public.h>
#include <Padico/Module.h>
#include <Padico/PadicoControl.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <Padico/PMI2.h>

struct nm_connector_pmi2_s
{
  int addr_len;
};

static void*nm_connector_pmi2_instantiate(puk_instance_t instance, puk_context_t context);
static void nm_connector_pmi2_destroy(void*_status);

static const struct puk_component_driver_s nm_connector_pmi2_component =
  {
    .instantiate = &nm_connector_pmi2_instantiate,
    .destroy     = &nm_connector_pmi2_destroy
  };

static void nm_connector_pmi2_create(void*_status, int addr_len, const char**url);
static int  nm_connector_pmi2_exchange(void*_status,
                                       const char*local_connector_url, const char*remote_connector_url,
                                       const void*local_cnx_addr, void*remote_cnx_addr);

static const struct nm_connector_iface_s nm_connector_pmi2 =
  {
    .create   = &nm_connector_pmi2_create,
    .exchange = &nm_connector_pmi2_exchange
  };

PADICO_MODULE_COMPONENT(NewMad_Connector_pmi2,
  puk_component_declare("NewMad_Connector_pmi2",
			puk_component_provides("PadicoComponent", "component", &nm_connector_pmi2_component),
			puk_component_provides("NewMad_Connector", "connector", &nm_connector_pmi2)));

/* ********************************************************* */

static void*nm_connector_pmi2_instantiate(puk_instance_t instance, puk_context_t context)
{
  struct nm_connector_pmi2_s*p_connector_pmi2 = malloc(sizeof(struct nm_connector_pmi2_s));
  return p_connector_pmi2;
}

static void nm_connector_pmi2_destroy(void*_status)
{
  struct nm_connector_pmi2_s*p_connector_pmi2 = _status;
  free(p_connector_pmi2);
}

/* ********************************************************* */

static void nm_connector_pmi2_create(void*_status, int addr_len, const char**url)
{
  struct nm_connector_pmi2_s*p_connector_pmi2 = _status;
  p_connector_pmi2->addr_len = addr_len;
  padico_string_t s_url = padico_string_new();
  padico_string_printf(s_url, "%s:%p",
                       (const char*)padico_na_node_getuuid(padico_na_getlocalnode()),
                       p_connector_pmi2);
  *url = padico_string_get(s_url);
}

static int nm_connector_pmi2_exchange(void*_status,
                                      const char*local_connector_url, const char*remote_connector_url,
                                      const void*local_cnx_addr, void*remote_cnx_addr)
{
  int rank = padico_na_rank();
  struct nm_connector_pmi2_s*p_connector_pmi2 = _status;
  int size = p_connector_pmi2->addr_len;
  int service_name_size = strlen(local_connector_url) + strlen(remote_connector_url) + 2;
  char*enc_addr = puk_hex_encode(local_cnx_addr, &size, NULL);

  //Publish local addr
  char*local_service_name = padico_malloc(service_name_size);
  sprintf(local_service_name, "%s-%s", local_connector_url, remote_connector_url);
  padico_pmi2_kvs_publish(local_service_name, enc_addr);

  //Retrieve remote addr
  char remote_enc_addr[size+1];
  char*remote_service_name = padico_malloc(service_name_size);
  sprintf(remote_service_name, "%s-%s", remote_connector_url, local_connector_url);
  padico_pmi2_kvs_lookup(remote_service_name, remote_enc_addr, size + 1);

  //Decode remote addr
  void*remote_addr = puk_hex_decode(remote_enc_addr, &size, NULL);
  memcpy(remote_cnx_addr, remote_addr, size);

  //Unpublish remote addr, this only works becquse there is a pmi2 synchro in control_minidriver_bootstrap_connect
  //  padico_pmi2_kvs_unpublish(remote_service_name);

  padico_free(enc_addr);
  padico_free(remote_addr);
  padico_free(local_service_name);
  padico_free(remote_service_name);
  return 0;
}
