/*
 * NewMadeleine
 * Copyright (C) 2012-2020 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */


#ifndef NM_MPI_NOT_IMPLEMENTED_H
#define NM_MPI_NOT_IMPLEMENTED_H


#include "mpi.h"
#include "mpio.h"
/** \addtogroup mpi_interface */
/* @{ */

/** @name Functions: the implementation of these functions is missing but the prototypes are required to ensure compatibility with other tools/applications */
/* @{ */

int MPI_Bsend_init(const void*buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request*request);
int MPI_Buffer_attach(void*buffer, int size);
int MPI_Buffer_detach(void *buffer_addr, int *size);

int MPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int*newrank);

int MPI_Graph_create(MPI_Comm comm_old, int nnodes, const int indx[],const int edges[], int reorder, MPI_Comm*comm_graph);
int MPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int indx[], int edges[]);
int MPI_Graph_map(MPI_Comm comm_old, int nnodes, const int*index, const int*edges, int*newrank);
int MPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);
int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors);
int MPI_Graphdims_get(MPI_Comm comm, int*nnodes, int*nedges);

int MPI_Ibsend(const void*buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request*request);

int MPI_Iallgatherv(const void*sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[],
                    const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request*request);

int MPI_Ialltoallv(const void*sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype,
                   void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm,
                   MPI_Request*request);
int MPI_Igatherv(const void*sendbuf, int sendcount, MPI_Datatype sendtype,
                 void*recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype,
                 int root, MPI_Comm comm, MPI_Request*request);
int MPI_Iscatter(const void*sendbuf, int sendcount, MPI_Datatype sendtype,
                 void*recvbuf, int recvcount, MPI_Datatype recvtype, int root,
                 MPI_Comm comm, MPI_Request*request);
int MPI_Iscatterv(const void*sendbuf, const int sendcounts[], const int displs[],
                  MPI_Datatype sendtype, void*recvbuf, int recvcount,
                  MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request*request);


NM_MPI_PMPI(MPI_Bsend_init);
NM_MPI_PMPI(MPI_Buffer_attach);
NM_MPI_PMPI(MPI_Buffer_detach);
NM_MPI_PMPI(MPI_Cart_map);
NM_MPI_PMPI(MPI_Graph_get);
NM_MPI_PMPI(MPI_Graph_map);
NM_MPI_PMPI(MPI_Graph_neighbors);
NM_MPI_PMPI(MPI_Graph_neighbors_count);
NM_MPI_PMPI(MPI_Graphdims_get);
NM_MPI_PMPI(MPI_Ibsend);

NM_MPI_PMPI(MPI_Iallgatherv);
NM_MPI_PMPI(MPI_Ialltoallv);
NM_MPI_PMPI(MPI_Igatherv);
NM_MPI_PMPI(MPI_Iscatter);
NM_MPI_PMPI(MPI_Iscatterv);

#define MPI_ARGV_NULL (NULL)



/* @}*/
/* @}*/

#endif /* NM_MPI_NOT_IMPLEMENTED_H */
