/*
 * NewMadeleine
 * Copyright (C) 2006-2019 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef NM_ONESIDED_INTERFACE_H
#define NM_ONESIDED_INTERFACE_H

#include <nm_public.h>
#include <nm_sendrecv_interface.h>
#include <nm_onesided_private.h>

#include <stdint.h>

/* ********************************************************* */

/** an instance of the 'onesided' interface */
typedef struct nm_onesided_s*nm_onesided_t;

/** request type for 'onesided' interface */
typedef struct nm_onesided_request_s nm_onesided_request_t;

/** initialize the interface, using the given session.
 */
void nm_onesided_init(nm_session_t p_session, nm_onesided_t*pp_onesided);

void nm_onesided_finalize(nm_onesided_t p_onesided);

/** remotely write to destination memory; local data described by p_data; dest_addr is 
 * the address in the remote memory space
 */
void nm_onesided_iput(nm_onesided_t p_onesided, struct nm_gate_s*p_gate, 
                      struct nm_data_s*p_data, uintptr_t dest_addr, nm_onesided_request_t*p_req);

void nm_onesided_iget(nm_onesided_t p_onesided, struct nm_gate_s*p_gate,
                      struct nm_data_s*p_data, uintptr_t dest_addr, nm_onesided_request_t*p_req);

/** ensures all put operations to the remote node are completed */
void nm_onesided_ifence(nm_onesided_t p_onesided, struct nm_gate_s*p_gate, nm_onesided_request_t*p_req);

/** wait for this request completion */
void nm_onesided_req_wait(nm_onesided_request_t*p_req);

/** blocking version of iput */
void nm_onesided_put(nm_onesided_t p_onesided, struct nm_gate_s*p_gate, 
                      struct nm_data_s*p_data, uintptr_t dest_addr);

/** blocking version of iget */
void nm_onesided_get(nm_onesided_t p_onesided, struct nm_gate_s*p_gate,
                      struct nm_data_s*p_data, uintptr_t dest_addr);
/** blocking version of ifence */
void nm_onesided_fence(nm_onesided_t p_onesided, struct nm_gate_s*p_gate);


#endif /* NM_ONESIDED_INTERFACE_H */

