/* -*- Mode: C; c-basic-offset:2 ; -*- */
/*
 * Pioman: a Generic I/O Manager
 * Copyright (C) 2008-2018 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef PIOM_NOTHREAD_H
#define PIOM_NOTHREAD_H

#ifndef PIOM_CONFIG_H
#  error "Cannot include this file directly. Please inclued <pioman.h>."
#endif /* PIOM_CONFIG_H */

#ifdef PIOMAN_MULTITHREAD
#  error "inconsistency detected: PIOMAN_MULTITHREAD defined in piom_nothread.h"
#endif /* PIOMAN_MULTITHREAD */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>

/* ** base dummy types ************************************* */

#define piom_thread_t     int
#define PIOM_THREAD_NULL  0
#define PIOM_THREAD_SELF  1

/* ** spinlocks for no-thread ****************************** */

typedef int piom_spinlock_t;

static inline void piom_spin_init(piom_spinlock_t*lock)
{
  *lock = 0;
}
static inline void piom_spin_destroy(piom_spinlock_t*lock)
{
  assert(*lock == 0);
}
static inline void piom_spin_lock(piom_spinlock_t*lock)
{
  assert(*lock == 0);
  *lock = 1;
}
static inline void piom_spin_unlock(piom_spinlock_t*lock)
{
  assert(*lock == 1);
  *lock = 0;
}
static inline int piom_spin_trylock(piom_spinlock_t*lock)
{
  if(*lock == 0)
    {
      *lock = 1;
      return 1;
    }
  else
    {
      return 0;
    }
}

static inline void piom_spin_assert_locked(piom_spinlock_t*lock)
{
  assert(*lock != 0);
}

static inline void piom_spin_assert_notlocked(piom_spinlock_t*lock)
{
  assert(*lock == 0);
}


/* ** semaphores ******************************************* */

/** dummy non-threaded semaphore */
typedef int piom_sem_t;

static inline void piom_sem_P(piom_sem_t*sem)
{
  (*sem)--;
  while((*sem) < 0)
    {
      piom_ltask_schedule(PIOM_POLL_POINT_BUSY);
    }
}

static inline void piom_sem_V(piom_sem_t*sem)
{
  (*sem)++;
}

static inline void piom_sem_init(piom_sem_t*sem, int initial)
{
  (*sem) = initial;
}

#endif /* PIOM_NOTHREAD_H */
