/* -*- Mode: C; c-basic-offset:2 ; -*- */
/*
 * Pioman: a Generic I/O Manager
 * Copyright (C) 2001-2020 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef PIOM_LOG_H
#define PIOM_LOG_H

#include <stdio.h>
#include <stdlib.h>
#include <execinfo.h>
#include <Padico/Puk.h>

/** display message in verbose mode, not in quiet mode */
#define PIOM_DISP(str, ...)						\
  {									\
   if(getenv("PIOM_VERBOSE") != NULL)					\
     {									\
      fprintf(stderr, "# %s: pioman: " str,				\
	      padico_hostname(), ## __VA_ARGS__);			\
     }									\
  }

/** warning message, always displayed */
#define PIOM_WARN(str, ...)						\
  {									\
    fprintf(stderr, "# %s: pioman: WARNING- %s() ",			\
	    padico_hostname(), __func__);				\
    fprintf(stderr, str , ## __VA_ARGS__);				\
  }

/** fatal error */
#define PIOM_FATAL(str, ...)						\
  {									\
    fprintf(stderr, "\n# %s: pioman: FATAL- error in %s() %s:%d\n\t",	\
            padico_hostname(), __func__, __FILE__, __LINE__);           \
    fprintf(stderr, str, ## __VA_ARGS__);				\
    fprintf(stderr, "\n\n");						\
    void*buffer[100];                                                   \
    int nptrs = backtrace(buffer, 100);                                 \
    backtrace_symbols_fd(buffer, nptrs, STDERR_FILENO);                 \
    fflush(stderr);                                                     \
    abort();								\
  }


#endif /* PIOM_LOG_H */
